#!/usr/bin/python
# Filename: logging_analyzer.py
"""
logging_analyzer for NetLoggerMMDiag. Developed for MILab.

Author: Haotian Deng
"""

import datetime
import math
import os
import re
import shutil
import subprocess
import sys
import threading
import urllib
import urlparse

from android.broadcast import BroadcastReceiver
from jnius import autoclass
from mobile_insight.analyzer.analyzer import *
from service import mi2app_utils

__all__=["LoggingAnalyzer"]

class LoggingAnalyzer(Analyzer):

    def __init__(self, config):
        Analyzer.__init__(self)
        self.__MILabPluginName = 'NetLoggerMMDiag'
        self.add_source_callback(self.__msg_callback)

        self.__own_log_uploaded_dir = os.path.join(
                mi2app_utils.get_mobileinsight_plugin_path(),
                self.__MILabPluginName,
                "log", "uploaded")
        if not os.path.exists(self.__own_log_uploaded_dir):
            os.makedirs(self.__own_log_uploaded_dir)

        self.__is_use_wifi = False
        self.__task = "Unknown"
        if 'task' in config:
            self.__task = config['task']
        if 'is_use_wifi' in config and config['is_use_wifi'] == '1':
            self.__is_use_wifi = True

        self.__own_log_task_dir = os.path.join(
                mi2app_utils.get_mobileinsight_plugin_path(),
                self.__MILabPluginName,
                "log", self.__task)
        if not os.path.exists(self.__own_log_task_dir):
            os.makedirs(self.__own_log_task_dir)

        self.br = BroadcastReceiver(self.on_broadcast,
                actions=['MobileInsight.Main.StopService'])
        self.br.start()

        self.brStop = BroadcastReceiver(self.on_broadcastStop,
                actions=['MobileInsight.MILab.Stop'])
        self.brStop.start()

        self.brStart = BroadcastReceiver(self.on_broadcastStart,
                actions=['MobileInsight.MILab.Start'])
        self.brStart.start()

    def on_broadcastStop(self, context, intent):
        self.log_info("MobileInsight.MILab.Stop is received")
        self.source._pause_collection()
        self._check_orphan_log()

    def on_broadcastStart(self, context, intent):
        self.log_info("MobileInsight.MILab.Start is received")
        self.source._resume_collection()

    def on_broadcast(self, context, intent):
        '''
        This plugin is going to be stopped, finish closure work
        '''
        print "LoggingAnalyzer are going to be stopped"

        self.log_info("MobileInsight.Main.StopService is received")
        self._check_orphan_log()

        IntentClass = autoclass("android.content.Intent")
        intent = IntentClass()
        action = 'MobileInsight.Plugin.StopServiceAck'
        intent.setAction(action)
        try:
            mi2app_utils.pyService.sendBroadcast(intent)
        except Exception as e:
            import traceback
            self.log_error(str(traceback.format_exc()))

    def set_source(self,source):
        """
        Set the trace source. Enable the cellular signaling messages

        :param source: the trace source (collector).
        """
        Analyzer.set_source(self,source)

    def _check_orphan_log(self):
        '''
        Check if there is any orphan log left in cache folder
        '''
        dated_files = []
        mi2log_folder = os.path.join(mi2app_utils.get_cache_dir(), "mi2log")
        for subdir, dirs, files in os.walk(mi2log_folder):
            for f in files:
                fn = os.path.join(subdir, f)
                dated_files.append((os.path.getmtime(fn), fn))
        dated_files.sort()
        dated_files.reverse()
        for dated_file in dated_files:
            self.__orig_file = dated_file[1]
            self.log_info("LoggingAnalyzer: find orphan log: %s" % self.__orig_file)
            mi2app_utils.run_shell_cmd("chmod 644 %s" % self.__orig_file)
            self._save_log()
            self.log_info("mi2log file saved")

    # def _upload_log(self, logfile):
    #     if not mi2app_utils.get_wifi_status():
    #         return
    #     import ehp
    #     basicUrl = "http://164.107.116.49/"
    #     s = requests.Session()
    #     url = basicUrl + 'milab/cellularlogs/create/'
    #     r = s.get(url)
    #     csrf_token = r.cookies['csrftoken']

    #     dictModel = {}
    #     dictOperator = {}
    #     dictPlugin = {}
    #     dictTask = {}

    #     html = ehp.Html()
    #     dom = html.feed(r.content)
    #     for root, item in dom.find_with_root('select'):
    #         if "id_model" in str(root[0]):
    #             for root, option in item.find_with_root('option'):
    #                 txt = str(option)
    #                 matchObj = re.match(r'^<option value="(.*)" >(.*)</option>$', txt, re.M)
    #                 if matchObj:
    #                     dictModel[matchObj.group(2)] = matchObj.group(1)
    #         elif "id_operator" in str(root[0]):
    #             for root, option in item.find_with_root('option'):
    #                 txt = str(option)
    #                 matchObj = re.match(r'^<option value="(.*)" >(.*)</option>$', txt, re.M)
    #                 if matchObj:
    #                     dictOperator[matchObj.group(2)] = matchObj.group(1)
    #         elif "id_plugin" in str(root[0]):
    #             for root, option in item.find_with_root('option'):
    #                 txt = str(option)
    #                 matchObj = re.match(r'^<option value="(.*)" >(.*)</option>$', txt, re.M)
    #                 if matchObj:
    #                     dictPlugin[matchObj.group(2)] = matchObj.group(1)
    #         elif "id_task" in str(root[0]):
    #             for root, option in item.find_with_root('option'):
    #                 txt = str(option)
    #                 matchObj = re.match(r'^<option value="(.*)" >(.*)</option>$', txt, re.M)
    #                 if matchObj:
    #                     dictTask[matchObj.group(2)] = matchObj.group(1)

    #     print dictModel
    #     print dictOperator
    #     print dictPlugin
    #     print dictTask
    #     phone_info = mi2app_utils.get_phone_info().split('_')[1]
    #     operator_info = mi2app_utils.get_operator_info()
    #     if self.__MILabPluginName not in dictPlugin \
    #             or phone_info not in dictModel\
    #             or operator_info not in dictOperator:
    #         self.log_error("mi2log (%s, %s, %s) upload aborted" % (
    #             self.__MILabPluginName,
    #             phone_info,
    #             operator_info))
    #         return
    #     multipart_form_data = {
    #         'logfile': open(logfile, 'rb'),
    #             }
    #     inputData={'csrfmiddlewaretoken': csrf_token,
    #         'plugin': dictPlugin[self.__MILabPluginName],
    #         'model': dictModel[phone_info],
    #         'operator': dictOperator[operator_info],
    #         }
    #     if self.__task in dictTask:
    #         inputData['task'] = dictTask[self.__task]
    #     r2 = s.post(url,
    #             data=inputData,
    #             headers=dict(Referer=url), files=multipart_form_data)
    #     self.log_info("mi2log file uploaded")
    #     shutil.copyfile(logfile, os.path.join(self.__own_log_uploaded_dir, os.path.basename(logfile)))
    #     os.remove(logfile)

    def _save_log(self):
        self.__log_timestamp = \
                datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
        milog_base_name = "diag_log_%s_%s_%s.mi2log" % (\
                self.__log_timestamp, \
                mi2app_utils.get_phone_info(), \
                mi2app_utils.get_operator_info())
        milog_abs_name  = os.path.join(self.__own_log_task_dir, milog_base_name)
        if os.path.isfile(self.__orig_file):
            shutil.copyfile(self.__orig_file, milog_abs_name)
            os.remove(self.__orig_file)
            if self.__task == "Unknown":
                pass
                # self._upload_log(milog_abs_name)

    def __msg_callback(self,msg):
        if msg.type_id.find("new_diag_log") != -1:
            log_item = msg.data.decode()
            self.__orig_file = log_item.get("filename")
            self.log_info("A new msg come in: " + self.__orig_file)
            # FIXME (Zengwen): the change access command is a walkaround solution
            mi2app_utils.run_shell_cmd("chmod 644 %s" % self.__orig_file)
            self._save_log()
            self.log_info("mi2log file saved")
